/*globals define*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ep/ui/spinner-test',
    'jquery',
    'ep/ui/spinner',

    'ep/ui/form',
    'jquery/metaparse',
    'jquery/i18n',
    'testsuite/simulate',
    '$ready!'
], function (QUnit, fixture, $, ep) {

    QUnit.module("ep/ui/spinner", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("ep/ui/spinner", function () {

        $('.ep-js').metaparse();

        var text_c1 = ep(':input[name=text_c1]'),
            text_c2 = ep(':input[name=text_c2]'),
            text_d = ep(':input[name=text_d]'),
            stepUp = ep('.ep-uiSpinner-stepUp'),
            stepDown = ep('.ep-uiSpinner-stepDown');

        text_c1.val('');
        text_c2.val('');
        text_d.val('');
        $('table input[type!=submit]').val('');

        QUnit.equal(
            text_c1.uiSpinner('Instance') instanceof $.Widget,
            true,
            "uiSpinner widget was created and is instance of jQuery.Widget"
        );

        $.each(['_create', '_init', '_changeAttr', '_parseValue', '_spin', '_spinDate', '_start', '_stop', '_tooltipInit', '_unitInit', 'destroy'], function (i, fn) {

            QUnit.equal(
                $.isFunction($.ui.uiSpinner.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );

        });

        QUnit.equal(
            stepUp.length,
            3,
            "$('.ep-uiSpinner-stepUp').length"
        );

        QUnit.equal(
            stepDown.length,
            3,
            "$('.ep-uiSpinner-stepDown').length"
        );

        stepUp
            .simulate('mousedown')
            .simulate('mouseup');

        QUnit.equal(
            text_c1.val(),
            '5,05',
            "$(':input[name=text_c1]').val()"
        );


        QUnit.equal(
            text_c2.val(),
            '3,50',
            "$(':input[name=text_c2]').val()"
        );

        QUnit.equal(
            text_d.val(),
            '21.12.2010',
            "$(':input[name=text_d]').val()"
        );

        // define key events
        var evtKeyDown = $.Event('keydown', {keyCode: 38}),
            evtKeyUp = $.Event('keyup', {keyCode: 38}),
            evtKeyPress = $.Event('keypress', {keyCode: 38}),
            evtKeyDown40 = $.Event('keydown', {keyCode: 40}),
            evtKeyUp40 = $.Event('keyup', {keyCode: 40}),
            evtKeyPress40 = $.Event('keypress', {keyCode: 40});

        text_c1.trigger(evtKeyDown);
        text_c1.trigger(evtKeyUp);
        text_c1.trigger(evtKeyPress);

        // This test was commented out because it leads to an error on Selenium that can't be reproduced.
        // You can use it for local testing.
        /*QUnit.equal(
            $('table input[name=text_c1]').filter(':visible').is(':focus'),
            true,
            "$('table input[name=text_c1]').filter(':visible').is(':focus')"
        );*/

        // trigger mousedown for second spinner
        $('table input[name=text_c2]').parent().find('a:first').trigger('mousedown');

        setTimeout(function () {
            // trigger mouseup for second spinner after 500 milliseconds
            $('table input[name=text_c2]').parent().find('a:first').trigger('mouseup');

            // check if base value was increased
            QUnit.ok(Number($(':input[name=text_c2]').val().replace(",", ".")) >= 3.50, 'value increased');

            // set value to maximum and try to increase value
            text_c2.val($.i18n.formatNumber(5.4, "n"));
            text_c2.trigger(evtKeyDown);
            text_c2.trigger(evtKeyUp);
            text_c2.trigger(evtKeyPress);

            QUnit.ok(Number($(':input[name=text_c2]').val().replace(",", ".")) <= 5.4, 'lower/equal than maximum (5.4)');

            // set value to minimum and try to decrease value
            text_c2.val($.i18n.formatNumber(2.5, "n"));
            text_c2.trigger(evtKeyDown40);
            text_c2.trigger(evtKeyDown40);

            // $('table input[name=text_c2]').trigger(evtKeyDown40);
            QUnit.ok(Number($(':input[name=text_c2]').val().replace(",", ".")) >= 2.5, 'bigger/equal than minimum (2.5)');


            // set value to minimum and try to decrease value
            text_d.val(new Date(2010, 10, 21));
            text_d.trigger(evtKeyDown40);
            text_d.trigger(evtKeyPress40);
            text_d.trigger(evtKeyUp40);

            // set value to maximum and try to increase value
            text_d.val(new Date(1293750000000));
            text_d.trigger(evtKeyDown);
            text_d.trigger(evtKeyPress);
            //$('table input[name=text_d]').trigger(evtKeyUp);

            // check if epSpinner-object exists in DOM
            QUnit.equal(
                text_d.parent()[0].nodeName.toLowerCase(),
                'td',
                "ep.uiSpinner object exists"
            );
            // destroy epSpinner-object
            text_d.uiSpinner('destroy');

            // check if epSpinner-object doesn't exists in DOM anymore
            QUnit.equal(
                text_d.parent()[0].nodeName.toLowerCase(),
                'td',
                "ep.uiSpinner object destroyed"
            );

            QUnit.equal(
                text_d.data('epUiTextedit'),
                undefined,
                'uiSpinner widget was destroyed'
            );

            QUnit.start();

        }, 500);

    });
});